﻿--[[
	Fallen Souls Loot
	Raid loot allocation system.
	Valerien@Deepwood
	
	An implementation of the Fallen Souls DKP system. Modified from
	an implementation of the EPGP raid loot allocation system.
--]]

local addon, private = ...

-- Update frequency when raiding (in seconds)
-- DKP is adjusted at each interval.
local UpdateFreq = 60*1

-- Our main EPGP data
epgp = GuildEPGP:Create()

-- Default DKP price list
local DKPPriceList = {
	{"Minor (1/2 DKP)", 50},
	{"Major / Progression (All DKP)", 100},
}

-- Default config options
fsloot_x = 200
fsloot_y = 200
fsloot_width = 400
fsloot_height = 500
fsloot_visible = true

-- Coroutines
local updategrid_coroutine = nil
local yieldTime = 0.01

-- Our grid status icons
local StatusGreen = "gfx/icons/status_green.png"
local StatusAmber = "gfx/icons/status_orange.png"
local StatusRed = "gfx/icons/status_red.png"

-- Remember our sort order
local LastSort = 2 -- also default sort order

-- Window caption data
local AddonData = Inspect.Addon.Detail("fsloot")
local Caption = "Fallen Souls Loot " .. AddonData.toc.Version

-- Create main window
win = NewWindow("Main", Caption)
win:SetWidth(fsloot_width)
win:SetHeight(fsloot_height)
win:SetPoint("TOPLEFT", UIParent, "TOPLEFT", fsloot_x, fsloot_y)
win.timerActive = false

-- Create any dialogs
local promptDialog = NewDialog(win)

-- Rounding numbers
function round(num, places)
  local mult = 10^(places or 0)
  return math.floor(num * mult + 0.5) / mult
end

function UpdateGrid()
	if updategrid_coroutine == nil then
		updategrid_coroutine = coroutine.create(DoUpdateGrid)
	end
end

-- Update our grid with the EPGP data
function DoUpdateGrid()
	local yield = Inspect.Time.Real() + yieldTime
	
	win.grid:Clear()
	local players = nil
	-- Only display active players if we're raiding
	--if win.timerActive then
		--players = epgp:GetActivePlayers()
	--else
		players = epgp.players
	--end
	-- Get player list
	nump = #players
	numr = win.grid.numRows
	while numr < nump do
		win.grid:AddRow({"","0"}, false)
		numr = numr + 1
	end
	
	-- Add all the player data to the grid
	for i = 1, nump do
		if(Inspect.Time.Real() >= yield and updategrid_coroutine) then
			coroutine.yield()
			yield = Inspect.Time.Real() + yieldTime
		end

		player = players[i]
		
		if(player) then
			row = win.grid.rows[i]
			row:SetText(1, player.playerName)
			if player.calling then
				row:SetTextColour(1, ClassColours[player.calling])
			end
			row:SetText(2, round(player:GetDKP(), 2))
			--row:SetText(4, player:GetGP())
			--row:SetText(2, player:GetPR())
			for j = 2, 4 do
				--row:SetTextColour(j, {r=1, g=1, b=1, a=1})
			end
			-- Set status icon
			if player.standby then
				row:SetIcon(StatusAmber)
			elseif player.active then
				row:SetIcon(StatusGreen)
			else
				row:SetIcon(StatusRed)
			end
		end
	end
	if nump ~= 0 then
		win.grid:Resize()
	end
end

-- Sort function
function Sort(index)
	-- We must clear any selection here as we don't support sorting selections
	win.grid:ClearSelection()
	if not index then
		index = LastSort
	else
		LastSort = index
	end
	-- Define a comparason function for each column
	function orderName(a, b)
		return a.playerName < b.playerName
	end
	function orderDKP(a, b)
		return a:GetDKP() > b:GetDKP()
	end
	function orderGP(a, b)
		return a:GetGP() > b:GetGP()
	end
	function orderPR(a, b)
		return a:GetPR() > b:GetPR()
	end
	-- Put functions in column order
	compare = {orderName, orderDKP}
	--compare = {orderName, orderPR, orderDKP, orderGP}
	-- Do the sort
	table.sort(epgp.players, compare[index])
	UpdateGrid()
end

-- Grid header click event handler
function onHeaderClicked(index)
	Sort(index)
end

-- Some event handlers
function ButtonAddClick()
	raid = GetRaidMembers(true)
	-- Add players to guild epgp data
	for _, p in pairs(raid) do
		epgp:AddPlayer(p.name, p.calling)
	end
	if win.timerActive then
		UpdateActive()
	end
	UpdateGrid(win.grid, epgp)
end

-- Start/Stop raid timer
function TimerStart(text)
	EPGP.dkpPerHour = tonumber(text)
	win.grid:ClearSelection()
	-- Start/stop raiding
	win.timerActive = not win.timerActive
	if win.timerActive then
		win.lastFrameTime = Inspect.Time.Real()
		win.caption:SetText(Caption .. " [Raid Active]")
		-- Determine who is active and mark them
		UpdateActive()
	else
		win.caption:SetText(Caption)
		-- Coming out of raid mode, clear all "active" flags
		for i = 1, #epgp.players do
			epgp.players[i].active = false
		end		
	end
	UpdateGrid()
end

function ButtonTimerClick(text)
	if(not(win.timerActive)) then
		GetEntry("Enter the DKP multiplier:",
			TimerStart)
	else
		TimerStart(1)
	end
end

-- Determine who is active/standby and update the database status
function UpdateActive()
	active = GetRaidMembers(false)
	-- Iterate all players in the database, set active if they are in 
	-- our "active" list
	for i = 1, #epgp.players do
		player = epgp.players[i].playerName
		epgp.players[i].active = false
		-- Brute force search, there aren't too many...
		for j = 1, #active do
			if player == active[j].name then
				epgp.players[i].standby = false
				epgp.players[i].active = true
				break
			end
		end
	end
end

-- Mark players as standby
function ButtonStandbyClick()
	standby = win.grid:GetSelection(true)
	if #standby <= 0 then return end
	-- mark as standby
	for i = 1, #standby do
		epgp:SetStandbyStatus(standby[i], not epgp:GetStandbyStatus(standby[i]))
	end
	win.grid:ClearSelection()
	UpdateGrid()
end

-- Select all/none in grid
function ButtonSelectAllClick()
	standby = win.grid:GetSelection(true)
	if #standby <= 0 and not win.timerActive then
		-- Select all
		win.grid:SelectAll()
	elseif #standby <= 0 and win.timerActive then
		for i = 1, #epgp.players do
			if (epgp.players[i].active == true) or (epgp.players[i].standby == true) then
				win.grid:SelectRow(i)
			end
		end
	else
		-- Select none
		win.grid:ClearSelection()
	end

	UpdateGrid()
end

-- Delete players data
function DeleteSelection()
	nuke = win.grid:GetSelection(true)
	if #nuke <= 0 then return end
	-- delete
	for i = 1, #nuke do
		epgp:DeletePlayer(nuke[i])
	end
	win.grid:ClearSelection()
	UpdateGrid()
end

function ButtonDeleteClick()
	GetConfirmation("Really permanently delete selected players?", 
		DeleteSelection)
end

function ButtonAddEPClick()
	GetEntry("Enter the number of DKP Points to add:",
		DoAddDKP)
end

function ButtonAddGPClick()
	GetOption("Enter the gear type:",
		DoAddGP)
end

function DoAddDKP(text)
	-- Bail out if we don't have a valid number
	ep = tonumber(text)
	if not ep then return end
	-- Add ep to all selected players
	add = win.grid:GetSelection(true)
	if #add <= 0 then return end
	-- Add the EP
	for i = 1, #add do
		p = epgp:GetPlayer(add[i])
		if p then
			p:IncDKP(ep)
		end
	end
	Sort()	
end

function DoAddGP(text)
	-- Bail out if we don't have a valid number
	gp = tonumber(text)
	if not gp then return end
	-- Add gp to selected player
	add = win.grid:GetSelection(true)
	if #add ~= 1 then 
		GetConfirmation("Must select only *one* player when adding gear.", nil)
		return
	end
	-- Add the GP
	p = epgp:GetPlayer(add[1])
	if p then
		p:IncGP(1)
		p:SetDKP(p:GetDKP() - (p:GetDKP() * (gp * .01)))
	end
	Sort()
end

-- Show a confirmation dialog
-- Pass prompt and callback to call on OK clicked
function GetConfirmation(msg, ok)
	promptDialog:SetOKCallback(ok)	
	promptDialog:Confirm(msg)
end

-- Show a dialog prompting for user entry
function GetEntry(msg, ok)
	promptDialog:SetOKCallback(ok)	
	promptDialog:GetEntry(msg)
end

-- Show a dialog prompting for an option selection
function GetOption(msg, ok)
	promptDialog:SetOKCallback(ok)	
	promptDialog:GetOption(msg, DKPPriceList)
end

-- Add some EP to all currently active players
function IncrementRaidEP()
	-- Work out the amount of DKP per interval
	ep = (EPGP.dkpPerHour / 3600) * UpdateFreq
	-- Get all active players
	active = epgp:GetActivePlayers()
	-- Add this to all active players
	for _, p in ipairs(active) do
		p:IncDKP(ep)
	end
	UpdateGrid()
end

function getDataCallback(failure, message)
	 if(not(message == nil)) then
		  print("getDataCallback failed")
	 end
end

function dataReceived(target, segment, identifier, read, write, data)
	words = { }
	for w, sep in string.gsplit(data, "[:%s]", true) do
		table.insert(words, w)
	end
	
	for i = 1, #words, 3 do
		if ((words[i] == nil) or (words[i+1] == nil) or (words[i+2] == nil)) then
			break end
			
		if tonumber(words[i+1]) == 1 then
			np = epgp:AddPlayer(words[i], "cleric")
		elseif tonumber(words[i+1]) == 2 then
			np = epgp:AddPlayer(words[i], "mage")
		elseif tonumber(words[i+1]) == 3 then
			np = epgp:AddPlayer(words[i], "rogue")
		elseif tonumber(words[i+1]) == 4 then
			np = epgp:AddPlayer(words[i], "warrior")
		else
			np = epgp:AddPlayer(words[i], "")
		end
		
		np:SetDKP(tonumber(words[i+2]))
		--np:SetGP(tonumber(words[i+3]))
	end
	
	UpdateGrid(win.grid, epgp)
end

-- Saved variables have been loaded
function onVariablesLoaded(id)
	if id ~= "fsloot" then return end
	
	-- load the data from guild storage
	local me = Inspect.Unit.Detail("player")
	local target = me.name
	
	Command.Storage.Get(target, "guild", "fsloot", getDataCallback)
	table.insert(Event.Storage.Get, { dataReceived, "fsloot", "dataReceived" })
	
	-- Apply configuration options
	win:SetVisible(fsloot_visible)
	if not fsloot_visible then
		print(AddonData.toc.Version .. " Loaded. Type /fsloot to open window manually")
	else
		print(AddonData.toc.Version .. " Loaded.")
	end
	
	win:SetWidth(fsloot_width)
	win:SetHeight(fsloot_height)
	win:SetPoint("TOPLEFT", UIParent, "TOPLEFT", fsloot_x, fsloot_y)
	
	if fsloot_button_x == nil and fsloot_button_y == nil then
		button:ClearAll()
		button:SetPoint("CENTER", UIParent, "CENTER")
		fsloot_button_x = button:GetLeft()
		fsloot_button_y = button:GetTop()
	end
	
	button:ClearAll()
	button:SetPoint("TOPLEFT", UIParent, "TOPLEFT", fsloot_button_x, fsloot_button_y)

	UpdateGrid()
end

function setDataCallback(failure, message)
	 if(not(message == nil)) then
		  print("setDataCallback failed")
	 end
end

function ButtonSelectRefresh()
	-- load the data from guild storage
	local me = Inspect.Unit.Detail("player")
	local target = me.name
	
	for _, p in pairs(epgp.players) do
		epgp:DeletePlayer(p.playerName)
	end
	
	Command.Storage.Get(target, "guild", "fsloot", getDataCallback)
	table.insert(Event.Storage.Get, { dataReceived, "fsloot", "dataReceived" })
	
	UpdateGrid()
end

function ButtonSelectPublish()
	local saved_epgp = {}
	local output = ""
	for _, p in pairs(epgp.players) do
		table.insert(saved_epgp, p.playerName)
		if p.calling == "cleric" then
			table.insert(saved_epgp, 1)
		elseif p.calling == "mage" then
			table.insert(saved_epgp, 2)
		elseif p.calling == "rogue" then
			table.insert(saved_epgp, 3)
		elseif p.calling == "warrior" then
			table.insert(saved_epgp, 4)
		else
			table.insert(saved_epgp, 0)
		end
		table.insert(saved_epgp, tostring(p:GetDKP()))
		--table.insert(saved_epgp, tostring(p:GetGP()))
	end
	
	for i = 1, #saved_epgp, 3 do
		output = string.concat(output, saved_epgp[i],":", saved_epgp[i+1], ":", saved_epgp[i+2], "\n")
	end
	
	Command.Storage.Set("guild", "fsloot", "guild", "officer", output, setDataCallback)
end

-- About to save variables
function onVariablesSave(id)
	if id ~= "fsloot" then return end
	fsloot_x = win:GetLeft()
	fsloot_y = win:GetTop()
	fsloot_width = win:GetWidth()
	fsloot_height = win:GetHeight()
	fsloot_button_x = button:GetLeft()
	fsloot_button_y = button:GetTop()
end

-- Main Window closed
function onMainWindowClose()
	print("Closed fsloot Window, use /fsloot to show it again")
	win:SetVisible(false)
	fsloot_visible = false
end

function run_updategrid_coroutine()
	if(updategrid_coroutine) then
		if(coroutine.status(updategrid_coroutine) == "dead") then
			updategrid_coroutine = nil
		else
			assert(coroutine.resume(updategrid_coroutine))
		end
	end
end

-- Frame update event handler, used for timing
function onFrameUpdate()
	-- Give coroutine some time
	run_updategrid_coroutine()
	
	-- Bail out if we're not actively timing
	if not win.timerActive then return end
	if not win.lastFrameTime then win.lastFrameTime = 0 end
	-- Get the current time
	t = Inspect.Time.Real()
	if t > win.lastFrameTime + UpdateFreq then
		win.lastFrameTime = t
		IncrementRaidEP()
	end
end

-- Base Slash command handler
local function slashCommand(param)
	if param == "" then
		local visible = not win:GetVisible()
		win:SetVisible( visible )
		fsloot_visible = visible
	else
		local parts = {}
		for w in param:gmatch("%w+") do
			table.insert(parts, w)
		end
		if #parts >= 1 then
			ProcessCommand(parts)
		end
	end
end

-- Toolbar icons
win.toolbar:AddButton("addplayers.png", 
	"Import current raid members (and current target)", ButtonAddClick)
win.toolbar:AddButton("delete.png", 
	"Delete selected players", ButtonDeleteClick)
win.toolbar:AddButton("raid.png", 
	"Start/Stop raid timer", ButtonTimerClick)
win.toolbar:AddButton("addep.png", 
	"Add DKP to selected players", ButtonAddEPClick)
win.toolbar:AddButton("addgp.png", 
	"Add loot for selected player (Remove DKP)", ButtonAddGPClick)
win.toolbar:AddButton("standby.png", 
	"Toggle standby status of selected players", ButtonStandbyClick)
win.toolbar:AddButton("selectall.png", 
	"Select all / Raid & Standby (if timing) / None", ButtonSelectAllClick)
win.toolbar:AddButton("refresh.png", 
	"Refresh changes from guild storage", ButtonSelectRefresh)
win.toolbar:AddButton("publish.png", 
	"Publish changes to guild storage", ButtonSelectPublish)


-- Create our grid
win.grid = NewGrid(win.workspace, 4, 10)
win.grid:AddRow({"Name", "DKP"}, true)
win.grid:SetHeaderCallback(onHeaderClicked)
-- Hook close event
win:SetCloseCallback(onMainWindowClose)

function ButtonClick()
	local visible = not win:GetVisible()
	win:SetVisible( visible )
	fsloot_visible = visible
end

-- Create button
button = NewButton("visbutton.png", ButtonClick)

-- Global event handlers

table.insert(Event.Addon.SavedVariables.Load.End, 
	{onVariablesLoaded, "fsloot", "Saved vars loaded"})
table.insert(Event.Addon.SavedVariables.Save.Begin, 
	{onVariablesSave, "fsloot", "About to save vars"})
table.insert(Event.System.Update.Begin, 
	{onFrameUpdate, "fsloot", "Frame redraw event"})
table.insert (Command.Slash.Register("fsloot"), 
	{slashCommand, "fsloot", "Show/hide main window"})
